from typing import List, Optional

from fastapi import APIRouter, Depends

from watchmen_auth import PrincipalService
from watchmen_indicator_kernel.meta import ObjectiveReportService
from watchmen_indicator_surface.util import trans, trans_readonly
from watchmen_meta.common import ask_meta_storage, ask_snowflake_generator
from watchmen_model.admin import UserRole
from watchmen_model.common import TenantId
from watchmen_model.indicator.objective_report import ObjectiveReport
from watchmen_rest import get_admin_principal, get_console_principal
from watchmen_rest.util import raise_403, validate_tenant_id
from watchmen_utilities import is_blank

router = APIRouter()


def get_objective_report_service(principal_service: PrincipalService) -> ObjectiveReportService:
	return ObjectiveReportService(ask_meta_storage(), ask_snowflake_generator(), principal_service)


@router.post('/indicator/objective/report', tags=[UserRole.ADMIN], response_model=ObjectiveReport)
async def save_objective_report(
		objective_report: ObjectiveReport,
		principal_service: PrincipalService = Depends(get_admin_principal)) -> ObjectiveReport:
	validate_tenant_id(objective_report, principal_service)
	objective_report_service = get_objective_report_service(principal_service)

	# noinspection DuplicatedCode
	def action(an_objective_report: ObjectiveReport) -> ObjectiveReport:
		if objective_report_service.is_storable_id_faked(an_objective_report.objectiveReportId):
			objective_report_service.redress_storable_id(an_objective_report)
			# noinspection PyTypeChecker
			an_objective: ObjectiveReport = objective_report_service.create(an_objective_report)
		# synchronize objective to user groups
		else:
			# noinspection PyTypeChecker
			existing_objective_report: Optional[ObjectiveReport] = objective_report_service.find_by_id(
				an_objective_report.objectiveId)
			if existing_objective_report is not None:
				if existing_objective_report.tenantId != an_objective_report.tenantId:
					raise_403()

			# noinspection PyTypeChecker
			an_objective: ObjectiveReport = objective_report_service.update(an_objective_report)

		return an_objective

	return trans(objective_report_service, lambda: action(objective_report))


@router.get('/indicator/objective/report/name', tags=[UserRole.ADMIN], response_model=List[ObjectiveReport])
async def find_objectives_by_name(
		query_name: Optional[str], principal_service: PrincipalService = Depends(get_console_principal)
) -> List[ObjectiveReport]:
	objective_report_service = get_objective_report_service(principal_service)

	def action() -> List[ObjectiveReport]:
		tenant_id: TenantId = principal_service.get_tenant_id()
		if is_blank(query_name):
			# noinspection PyTypeChecker
			return objective_report_service.find_by_name(None, tenant_id)
		else:
			# noinspection PyTypeChecker
			return objective_report_service.find_by_name(query_name, tenant_id)

	return trans_readonly(objective_report_service, action)



from typing import List, Optional

from fastapi import APIRouter, Body, Depends

from watchmen_auth import PrincipalService
from watchmen_indicator_kernel.meta import IndicatorService
from watchmen_indicator_surface.settings import ask_tuple_delete_enabled
from watchmen_indicator_surface.util import trans, trans_readonly
from watchmen_lineage.service.lineage_service import LineageService
from watchmen_meta.admin import UserService
from watchmen_meta.common import ask_meta_storage, ask_snowflake_generator
from watchmen_model.admin import UserRole
from watchmen_model.common import DataPage, IndicatorId, Pageable, TenantId
from watchmen_model.indicator import Indicator
from watchmen_rest import get_admin_principal, get_console_principal, get_super_admin_principal
from watchmen_rest.util import raise_400, raise_403, raise_404, validate_tenant_id
from watchmen_utilities import is_blank

router = APIRouter()

lineage_service = LineageService()


def get_indicator_service(principal_service: PrincipalService) -> IndicatorService:
	return IndicatorService(ask_meta_storage(), ask_snowflake_generator(), principal_service)


def get_user_service(indicator_service: IndicatorService) -> UserService:
	return UserService(
		indicator_service.storage, indicator_service.snowflakeGenerator, indicator_service.principalService)


@router.get('/indicator/indicator/name', tags=[UserRole.CONSOLE, UserRole.ADMIN], response_model=List[Indicator])
async def find_indicators_by_name(
		query_name: Optional[str], principal_service: PrincipalService = Depends(get_console_principal)
) -> List[Indicator]:
	indicator_service = get_indicator_service(principal_service)

	def action() -> List[Indicator]:
		tenant_id: TenantId = principal_service.get_tenant_id()
		if is_blank(query_name):
			# noinspection PyTypeChecker
			return indicator_service.find_by_text(None, tenant_id)
		else:
			# noinspection PyTypeChecker
			return indicator_service.find_by_text(query_name, tenant_id)

	return trans_readonly(indicator_service, action)


class QueryIndicatorDataPage(DataPage):
	data: List[Indicator]


@router.post('/indicator/indicator/name', tags=[UserRole.ADMIN], response_model=QueryIndicatorDataPage)
async def find_indicators_page_by_name(
		query_name: Optional[str], pageable: Pageable = Body(...),
		principal_service: PrincipalService = Depends(get_admin_principal)) -> QueryIndicatorDataPage:
	indicator_service = get_indicator_service(principal_service)

	def action() -> QueryIndicatorDataPage:
		tenant_id: TenantId = principal_service.get_tenant_id()
		if is_blank(query_name):
			# noinspection PyTypeChecker
			return indicator_service.find_page_by_text(None, tenant_id, pageable)
		else:
			# noinspection PyTypeChecker
			return indicator_service.find_page_by_text(query_name, tenant_id, pageable)

	return trans_readonly(indicator_service, action)


@router.get('/indicator/indicator/all', tags=[UserRole.CONSOLE, UserRole.ADMIN], response_model=List[Indicator])
async def find_all_indicators(principal_service: PrincipalService = Depends(get_console_principal)) -> List[Indicator]:
	indicator_service = get_indicator_service(principal_service)

	def action() -> List[Indicator]:
		tenant_id: TenantId = principal_service.get_tenant_id()
		return indicator_service.find_all(tenant_id)

	return trans_readonly(indicator_service, action)


@router.get('/indicator/indicator', tags=[UserRole.ADMIN], response_model=Indicator)
async def load_indicator_by_id(
		indicator_id: Optional[IndicatorId], principal_service: PrincipalService = Depends(get_admin_principal)
) -> Indicator:
	if is_blank(indicator_id):
		raise_400('Indicator id is required.')

	indicator_service = get_indicator_service(principal_service)

	def action() -> Indicator:
		# noinspection PyTypeChecker
		indicator: Indicator = indicator_service.find_by_id(indicator_id)
		if indicator is None:
			raise_404()
		# tenant id must match current principal's
		if indicator.tenantId != principal_service.get_tenant_id():
			raise_404()

		return indicator

	return trans_readonly(indicator_service, action)


@router.post('/indicator/indicator', tags=[UserRole.ADMIN], response_model=Indicator)
async def save_indicator(
		indicator: Indicator, principal_service: PrincipalService = Depends(get_admin_principal)) -> Indicator:
	validate_tenant_id(indicator, principal_service)
	indicator_service = get_indicator_service(principal_service)

	# noinspection DuplicatedCode
	def action(an_indicator: Indicator) -> Indicator:
		if indicator_service.is_storable_id_faked(an_indicator.indicatorId):
			indicator_service.redress_storable_id(an_indicator)
			# noinspection PyTypeChecker
			an_indicator: Indicator = indicator_service.create(an_indicator)
		else:
			# noinspection PyTypeChecker
			existing_indicator: Optional[Indicator] = indicator_service.find_by_id(an_indicator.indicatorId)
			if existing_indicator is not None:
				if existing_indicator.tenantId != an_indicator.tenantId:
					raise_403()

			# noinspection PyTypeChecker
			an_indicator: Indicator = indicator_service.update(an_indicator)

		return an_indicator

	return trans(indicator_service, lambda: action(indicator))


@router.get('/indicator/indicator/relevant', tags=[UserRole.ADMIN], response_model=List[Indicator])
async def find_relevant_indicators(
		indicator_id: Optional[IndicatorId], principal_service: PrincipalService = Depends(get_admin_principal)
) -> List[Indicator]:
	if is_blank(indicator_id):
		raise_400('Indicator id is required.')

	indicator_service = get_indicator_service(principal_service)
	## TODO remove after build_partial_lineage
	lineage_service.init_tenant_all_lineage_data(principal_service)
	def action() -> List[Indicator]:
		# noinspection PyTypeChecker
		indicator: Indicator = indicator_service.find_by_id(indicator_id)
		if indicator is None:
			raise_404()
		# tenant id must match current principal's
		if indicator.tenantId != principal_service.get_tenant_id():
			raise_404()

		indicator_id_list = lineage_service.load_relevant_indicators(indicator.indicatorId,principal_service)
		return indicator_service.find_by_ids(indicator_id_list,principal_service.get_tenant_id())

	return trans_readonly(indicator_service, action)


@router.post('/indicator/indicator/category/available', tags=[UserRole.ADMIN], response_model=List[str])
async def load_indicator_category(
		prefix: List[str], principal_service: PrincipalService = Depends(get_admin_principal)) -> List[str]:
	indicator_service = get_indicator_service(principal_service)

	def action() -> List[str]:
		# noinspection PyTypeChecker
		return indicator_service.find_available_categories(prefix, principal_service.get_tenant_id())

	return trans_readonly(indicator_service, action)


@router.get('/indicator/indicator/export', tags=[UserRole.ADMIN], response_model=List[Indicator])
async def find_buckets_for_export(
		principal_service: PrincipalService = Depends(get_admin_principal)) -> List[Indicator]:
	indicator_service = get_indicator_service(principal_service)

	def action() -> List[Indicator]:
		tenant_id: TenantId = principal_service.get_tenant_id()
		return indicator_service.find_all(tenant_id)

	return trans_readonly(indicator_service, action)


@router.delete('/indicator/indicator', tags=[UserRole.SUPER_ADMIN], response_model=Indicator)
async def delete_indicator_by_id_by_super_admin(
		indicator_id: Optional[IndicatorId] = None,
		principal_service: PrincipalService = Depends(get_super_admin_principal)
) -> Indicator:
	if not ask_tuple_delete_enabled():
		raise_404('Not Found')

	if is_blank(indicator_id):
		raise_400('Indicator id is required.')

	indicator_service = get_indicator_service(principal_service)

	def action() -> Indicator:
		# noinspection PyTypeChecker
		indicator: Indicator = indicator_service.delete(indicator_id)
		if indicator is None:
			raise_404()
		return indicator

	return trans(indicator_service, action)



from typing import List, Optional, Tuple

from fastapi import APIRouter, Body, Depends
from pydantic import BaseModel

from watchmen_auth import PrincipalService
from watchmen_indicator_kernel.meta import BucketService
from watchmen_indicator_surface.settings import ask_tuple_delete_enabled
from watchmen_indicator_surface.util import trans, trans_readonly
from watchmen_meta.common import ask_meta_storage, ask_snowflake_generator
from watchmen_model.admin import UserRole
from watchmen_model.common import BucketId, DataPage, EnumId, Pageable, TenantId
from watchmen_model.indicator import Bucket, MeasureMethod
from watchmen_rest import get_admin_principal, get_console_principal, get_super_admin_principal
from watchmen_rest.util import raise_400, raise_403, raise_404, validate_tenant_id
from watchmen_utilities import ArrayHelper, is_blank, is_not_blank

router = APIRouter()


def get_bucket_service(principal_service: PrincipalService) -> BucketService:
	return BucketService(ask_meta_storage(), ask_snowflake_generator(), principal_service)


@router.get('/indicator/bucket', tags=[UserRole.CONSOLE, UserRole.ADMIN], response_model=Bucket)
async def load_bucket_by_id(
		bucket_id: Optional[BucketId], principal_service: PrincipalService = Depends(get_console_principal)
) -> Bucket:
	if is_blank(bucket_id):
		raise_400('Bucket id is required.')

	bucket_service = get_bucket_service(principal_service)

	def action() -> Bucket:
		# noinspection PyTypeChecker
		bucket: Bucket = bucket_service.find_by_id(bucket_id)
		if bucket is None:
			raise_404()
		# tenant id must match current principal's
		if bucket.tenantId != principal_service.get_tenant_id():
			raise_404()
		return bucket

	return trans_readonly(bucket_service, action)


@router.post('/indicator/bucket', tags=[UserRole.ADMIN], response_model=Bucket)
async def save_bucket(bucket: Bucket, principal_service: PrincipalService = Depends(get_admin_principal)) -> Bucket:
	validate_tenant_id(bucket, principal_service)
	bucket_service = get_bucket_service(principal_service)

	# noinspection DuplicatedCode
	def action(a_bucket: Bucket) -> Bucket:
		if bucket_service.is_storable_id_faked(a_bucket.bucketId):
			bucket_service.redress_storable_id(a_bucket)
			# noinspection PyTypeChecker
			a_bucket: Bucket = bucket_service.create(a_bucket)
		else:
			# noinspection PyTypeChecker
			existing_bucket: Optional[Bucket] = bucket_service.find_by_id(a_bucket.bucketId)
			if existing_bucket is not None:
				if existing_bucket.tenantId != a_bucket.tenantId:
					raise_403()

			# noinspection PyTypeChecker
			a_bucket: Bucket = bucket_service.update(a_bucket)

		return a_bucket

	return trans(bucket_service, lambda: action(bucket))


class QueryBucketDataPage(DataPage):
	data: List[Bucket]


@router.post('/indicator/bucket/name', tags=[UserRole.ADMIN], response_model=QueryBucketDataPage)
async def find_buckets_page_by_name(
		query_name: Optional[str], pageable: Pageable = Body(...),
		principal_service: PrincipalService = Depends(get_admin_principal)) -> QueryBucketDataPage:
	bucket_service = get_bucket_service(principal_service)

	def action() -> QueryBucketDataPage:
		tenant_id: TenantId = principal_service.get_tenant_id()
		if is_blank(query_name):
			# noinspection PyTypeChecker
			return bucket_service.find_page_by_text(None, tenant_id, pageable)
		else:
			# noinspection PyTypeChecker
			return bucket_service.find_page_by_text(query_name, tenant_id, pageable)

	return trans_readonly(bucket_service, action)


@router.get("/indicator/bucket/numeric-value", tags=[UserRole.CONSOLE, UserRole.ADMIN], response_model=List[Bucket])
async def find_buckets_by_numeric_value(
		query_name: Optional[str], principal_service: PrincipalService = Depends(get_console_principal)) -> List[
	Bucket]:
	bucket_service = get_bucket_service(principal_service)

	def action() -> List[Bucket]:
		tenant_id: TenantId = principal_service.get_tenant_id()
		if is_blank(query_name):
			# noinspection PyTypeChecker
			return bucket_service.find_numeric_value_by_text(None, tenant_id)
		else:
			# noinspection PyTypeChecker
			return bucket_service.find_numeric_value_by_text(query_name, tenant_id)

	return trans_readonly(bucket_service, action)


class QueryByBucketMethod(BaseModel):
	method: MeasureMethod = None
	enumId: Optional[EnumId] = None


class QueryByBucket(BaseModel):
	methods: List[QueryByBucketMethod]


@router.post("/indicator/bucket/methods", tags=[UserRole.CONSOLE, UserRole.ADMIN], response_model=List[Bucket])
async def find_buckets_by_measure_methods(
		query_bucket: QueryByBucket, principal_service: PrincipalService = Depends(get_console_principal)
) -> List[Bucket]:
	if query_bucket is None:
		raise_400('Measure method is required.')
	if query_bucket.methods is None or len(query_bucket.methods) == 0:
		raise_400('Measure method is required.')
	measure_methods: List[Tuple[MeasureMethod, Optional[EnumId]]] = ArrayHelper(query_bucket.methods) \
		.filter(lambda x: x is not None and x.method is not None) \
		.filter(lambda x: x.method != MeasureMethod or (x.method == MeasureMethod.ENUM and is_not_blank(x.enumId))) \
		.map(lambda x: (x.method, x.enumId)) \
		.to_list()
	if len(measure_methods) == 0:
		raise_400('Measure method is required.')

	bucket_service = get_bucket_service(principal_service)

	def action() -> List[Bucket]:
		tenant_id: TenantId = principal_service.get_tenant_id()
		return bucket_service.find_by_measure_method(measure_methods, tenant_id)

	return trans_readonly(bucket_service, action)


@router.post('/indicator/bucket/ids', tags=[UserRole.CONSOLE, UserRole.ADMIN], response_model=List[Bucket])
async def find_buckets_by_measure_methods(
		bucket_ids: List[BucketId], principal_service: PrincipalService = Depends(get_console_principal)
) -> List[Bucket]:
	if len(bucket_ids) == 0:
		return []

	bucket_service = get_bucket_service(principal_service)

	def action() -> List[Bucket]:
		tenant_id: TenantId = principal_service.get_tenant_id()
		return bucket_service.find_by_ids(bucket_ids, tenant_id)

	return trans_readonly(bucket_service, action)


@router.get('/indicator/bucket/export', tags=[UserRole.ADMIN], response_model=List[Bucket])
async def find_buckets_for_export(principal_service: PrincipalService = Depends(get_admin_principal)) -> List[Bucket]:
	bucket_service = get_bucket_service(principal_service)

	def action() -> List[Bucket]:
		tenant_id: TenantId = principal_service.get_tenant_id()
		return bucket_service.find_all(tenant_id)

	return trans_readonly(bucket_service, action)


@router.delete('/indicator/bucket', tags=[UserRole.SUPER_ADMIN], response_model=Bucket)
async def delete_bucket_by_id_by_super_admin(
		bucket_id: Optional[BucketId] = None,
		principal_service: PrincipalService = Depends(get_super_admin_principal)
) -> Bucket:
	if not ask_tuple_delete_enabled():
		raise_404('Not Found')

	if is_blank(bucket_id):
		raise_400('Bucket id is required.')

	bucket_service = get_bucket_service(principal_service)

	def action() -> Bucket:
		# noinspection PyTypeChecker
		bucket: Bucket = bucket_service.delete(bucket_id)
		if bucket is None:
			raise_404()
		return bucket

	return trans(bucket_service, action)



from fastapi import APIRouter

# from watchmen_auth import PrincipalService
# from watchmen_indicator_kernel.meta import AchievementPluginTaskService
# from watchmen_indicator_kernel.plugin import call_connector_service
# from watchmen_indicator_surface.util import trans, trans_readonly
# from watchmen_meta.common import ask_meta_storage, ask_snowflake_generator
# from watchmen_meta.system import PluginService
# from watchmen_model.admin import UserRole
# from watchmen_model.common import AchievementPluginTaskId, PluginId
# from watchmen_model.indicator import AchievementPluginTask
# from watchmen_model.indicator.achievement_plugin_task import AchievementPluginTaskStatus
# from watchmen_model.system import Plugin, PluginApplyTo
# from watchmen_rest import get_any_admin_principal, get_console_principal
# from watchmen_rest.util import raise_400, raise_404
# from watchmen_utilities import is_blank

router = APIRouter()
# TODO REFACTOR-OBJECTIVE ACHIEVEMENT TASKS
# def get_task_service(principal_service: PrincipalService) -> AchievementPluginTaskService:
# 	return AchievementPluginTaskService(ask_meta_storage(), ask_snowflake_generator(), principal_service)
#
#
# def get_achievement_service(task_service: AchievementPluginTaskService) -> AchievementService:
# 	return AchievementService(task_service.storage, task_service.snowflakeGenerator, task_service.principalService)
#
#
# def get_plugin_service(task_service: AchievementPluginTaskService) -> PluginService:
# 	return PluginService(task_service.storage, task_service.snowflakeGenerator, task_service.principalService)
#
#
# def ask_create_task_action(
# 		task_service: AchievementPluginTaskService, principal_service: PrincipalService
# ) -> Callable[[AchievementId, PluginId], Tuple[AchievementPluginTask, Plugin]]:
# 	# noinspection DuplicatedCode
# 	def action(achievement_id: AchievementId, plugin_id: PluginId) -> Tuple[AchievementPluginTask, Plugin]:
# 		achievement_service = get_achievement_service(task_service)
# 		objective_factor: Optional[Achievement] = achievement_service.find_by_id(achievement_id)
# 		if objective_factor is None:
# 			raise_404('Achievement not found.')
# 		if objective_factor.tenantId != principal_service.get_tenant_id():
# 			raise_404('Achievement not found.')
#
# 		plugin_service = get_plugin_service(task_service)
# 		plugin: Optional[Plugin] = plugin_service.find_by_id(plugin_id)
# 		if plugin is None:
# 			raise_404('Achievement plugin not found.')
# 		if plugin.tenantId != principal_service.get_tenant_id():
# 			raise_404('Achievement plugin not found.')
# 		if plugin.applyTo != PluginApplyTo.ACHIEVEMENT:
# 			raise_400('Plugin is not for objective_factor.')
#
# 		task = AchievementPluginTask(
# 			achievementId=achievement_id,
# 			pluginId=plugin_id,
# 			status=AchievementPluginTaskStatus.SUBMITTED,
# 			userId=principal_service.get_user_id(),
# 			tenantId=principal_service.get_tenant_id()
# 		)
# 		task_service.redress_storable_id(task)
# 		# noinspection PyTypeChecker
# 		task: AchievementPluginTask = task_service.create(task)
#
# 		return task, plugin
#
# 	return action
#
#
# @router.post(
# 	'/indicator/objective_factor/task', tags=[UserRole.CONSOLE, UserRole.ADMIN], response_model=AchievementPluginTask)
# def create_task(
# 		achievement_id: Optional[AchievementId], plugin_id: Optional[PluginId],
# 		principal_service: PrincipalService = Depends(get_console_principal)
# ) -> AchievementPluginTask:
# 	if is_blank(achievement_id):
# 		raise_400('Achievement id is required.')
# 	if is_blank(plugin_id):
# 		raise_400('Plugin id is required.')
# 	# principal_service.
# 	task_service = get_task_service(principal_service)
# 	action = ask_create_task_action(task_service, principal_service)
# 	task, plugin = trans(task_service, lambda: action(achievement_id, plugin_id))
# 	call_connector_service(task, plugin)
# 	return task
#
#
# @router.get(
# 	'/indicator/objective_factor/task', tags=[UserRole.CONSOLE, UserRole.ADMIN], response_model=AchievementPluginTask)
# def check_task_status(
# 		task_id: Optional[AchievementPluginTaskId],
# 		principal_service: PrincipalService = Depends(get_console_principal)
# ) -> AchievementPluginTask:
# 	if is_blank(task_id):
# 		raise_400('Achievement plugin task id is required.')
#
# 	task_service = get_task_service(principal_service)
#
# 	def action() -> AchievementPluginTask:
# 		# noinspection PyTypeChecker
# 		task: AchievementPluginTask = task_service.find_by_id(task_id)
# 		if task is None:
# 			raise_404()
# 		# tenant id must match current principal's
# 		if task.tenantId != principal_service.get_tenant_id():
# 			raise_404()
# 		if task.userId != principal_service.get_user_id():
# 			raise_404()
# 		return task
#
# 	return trans_readonly(task_service, action)
#
#
# class TaskResult(BaseModel):
# 	taskId: AchievementPluginTaskId
# 	status: AchievementPluginTaskStatus
# 	url: Optional[str]
#
#
# @router.post(
# 	'/indicator/objective_factor/task/result', tags=[UserRole.ADMIN, UserRole.SUPER_ADMIN], response_class=Response)
# def write_back_task_result(
# 		result: TaskResult,
# 		principal_service: PrincipalService = Depends(get_any_admin_principal)
# ) -> None:
# 	if is_blank(result.taskId):
# 		raise_400('Achievement plugin task id of result is required.')
# 	if result.status is None:
# 		raise_400('Achievement plugin task status of result is required.')
# 	if result.status == AchievementPluginTaskStatus.SUCCESS and is_blank(result.url):
# 		raise_400('Achievement plugin task url of result is required when successful.')
#
# 	task_service = get_task_service(principal_service)
#
# 	def action() -> None:
# 		# noinspection PyTypeChecker
# 		task: Optional[AchievementPluginTask] = task_service.find_by_id(result.taskId)
# 		if task is None:
# 			raise_404()
# 		if principal_service.is_tenant_admin() and task.tenantId != principal_service.get_tenant_id():
# 			raise_404()
# 		task.status = result.status
# 		task.url = result.url
# 		task_service.update(task)
#
# 	trans(task_service, action)



from logging import getLogger

from pydantic import BaseSettings

logger = getLogger(__name__)


class IndicatorSurfaceSettings(BaseSettings):
	TUPLE_DELETABLE: bool = False

	class Config:
		# secrets_dir = '/var/run'
		env_file = '.env'
		env_file_encoding = 'utf-8'
		case_sensitive = True


settings = IndicatorSurfaceSettings()


def ask_tuple_delete_enabled() -> bool:
	return settings.TUPLE_DELETABLE



from logging import getLogger
from typing import Optional

from fastapi import APIRouter, Depends

from watchmen_auth import PrincipalService
from watchmen_indicator_kernel.data import ConvergenceData, get_convergence_data_service
from watchmen_indicator_kernel.meta import ConvergenceService
from watchmen_meta.common import ask_meta_storage, ask_snowflake_generator
from watchmen_model.admin import UserRole
from watchmen_model.common import ConvergenceId
from watchmen_model.indicator import Convergence
from watchmen_rest import get_admin_principal
from watchmen_rest.util import raise_400, raise_403, raise_404
from watchmen_utilities import is_blank

logger = getLogger(__name__)
router = APIRouter()


def get_convergence_service(principal_service: PrincipalService) -> ConvergenceService:
	return ConvergenceService(ask_meta_storage(), ask_snowflake_generator(), principal_service)


@router.post('/indicator/convergence/data', tags=[UserRole.ADMIN], response_model=ConvergenceData)
async def load_convergence_data(
		convergence_id: Optional[ConvergenceId] = None,
		principal_service: PrincipalService = Depends(get_admin_principal)) -> ConvergenceData:
	if is_blank(convergence_id):
		raise_400('Convergence id is required.')

	convergence_service = get_convergence_service(principal_service)
	# noinspection PyTypeChecker
	convergence: Convergence = convergence_service.find_by_id(convergence_id)
	if convergence is None:
		raise_404()

	if convergence.tenantId != principal_service.get_tenant_id():
		raise_403()

	convergence_data_service = get_convergence_data_service(convergence, principal_service)
	return convergence_data_service.ask_values()
